function [A] = hf(filenameIn,hdeg)

  clc;

  %filenameIn = 'sample_points';
  %hdeg = 10; % highest degree of HF to find

  fprintf(['===============================\n']);
  fprintf(['**Using file: ',filenameIn,' **\n\n']);
  [n,Y] = ReadInStartAuto(filenameIn);
  %Y1 = Y;
  Y1 = [ones(size(Y,1),1),Y];
  A = HilbertPoints(Y1,1e-8,hdeg);
  Acopy = A;
  fprintf(['===============================\nHilbert data: (based on ',num2str(n),' points)\n']);
  disp(A');
  fprintf('Computing first difference...\n');
  A = [1;diff(A)];
  disp(A');

return;


function [HF, SV] = HilbertPoints(Points, tol, maxDeg)

% Jonathan D. Hauenstein
% Copyright 2011
% Compute the Hilbert function of a set of points in projective space

    % find the number of Points
    numPoints = size(Points, 1);

    % allocate HF
    HF = zeros(maxDeg+1,1);

    % compute intrinsic coordinates
    V = VeroneseMatrix(Points, 1);
    N = null(V);
    if size(N,2) == 0
        % full rank
        Int = Points;
    else
        % work intrinsically
        N = null(transpose(N));
        Int = transpose(N \ transpose(Points));
    end;
    
    % compute the Hilbert function using the intrinsic coordinates
    HF(1) = 1;
    SV = struct('maxSV',1,'minNonzeroSV',1,'maxZeroSV',0);
    disp('HF[0] = 1');
    
    for degree = 1:maxDeg
        if degree > 1 && HF(degree) == numPoints
            HF(degree+1) = numPoints;
            disp(['HF[',num2str(degree),'] = ',num2str(HF(degree+1))]);
        elseif degree > 1 && HF(degree) == HF(degree-1)
            HF(degree+1) = HF(degree);
            disp(['HF[',num2str(degree),'] = ',num2str(HF(degree+1))]);
        else
%disp('setup VM');
            V = VeroneseMatrix(Int, degree);
%disp('SVD');
            S = svd(V);
     
            HF(degree+1) = sum(S > tol);
            SV.maxSV(degree+1) = S(1);
            SV.minNonzeroSV(degree+1) = S(HF(degree+1));
            if HF(degree+1) < size(S,1);
                SV.maxZeroSV(degree+1) = S(HF(degree+1)+1);
            else
                SV.maxZeroSV(degree+1) = 0;
            end;
            disp(['HF[',num2str(degree),'] = ',num2str(HF(degree+1)),'  (max: ',...
                num2str(SV.maxSV(degree+1)),' min: ',num2str(SV.minNonzeroSV(degree+1)),' zero: ',num2str(SV.maxZeroSV(degree+1)),')']);
            
            if HF(degree+1) < HF(degree)
                disp(' ');
                disp(['Error: Since the Hilbert function is nondecreasing, there is an error in degree ',num2str(degree),'!']);
                return;
            end;
        end;
    end;

return;
                   

function [numPts, Pts] = LoadPoints(fileName, numVars)

% Jonathan D. Hauenstein
% Copyright 2011
% Load points stored in the file named fileName with numVars number of
% coordinates

    % open file
    RF = fopen(fileName, 'r');

    % make sure the file exists
    if RF == -1
        % file does not exist
        disp(['The file ''', fileName, ''' does not exist!']);
        numPts = 0;
        Pts = 0;
        return;
    end;

    % read in the number of points
    numPts = fscanf(RF, '%d', 1);

    % setup Pts
    Pts = zeros(numPts, numVars);

    % loop through to read in the points
    for k = 1:numPts
if mod(k,10000) == 1 disp([num2str(k),' of ',num2str(numPts)]); end;
        % read in the point
        for j = 1:numVars
            P = fscanf(RF, '%e', 2);
            Pts(k,j) = P(1) + 1i*P(2);
        end;
    end;

    % close file
    fclose(RF);

return;


function [numPts, Pts] = ReadInStartAuto(fileLocation)

  % Determine number of coordinates
  fileIn = fopen(fileLocation,'r');
  textline = fgetl(fileIn);
  numPoints = str2num(textline);
  textline = fgetl(fileIn);
  numVars = -1;
  while(strcmp(textline,'')==0 || numVars==-1)
    textline = fgetl(fileIn);
    numVars = numVars + 1;
  end;
  fclose(fileIn);
  fprintf(['Reading file: ',fileLocation,'\n']);
  fprintf(['Found ',num2str(numPoints),' points in ',num2str(numVars),' coordinates.\n']);

  % open file
  RF = fopen(fileLocation, 'r');

  % make sure the file exists
  if RF == -1
      % file does not exist
      disp(['The file ''', fileLocation, ''' does not exist!']);
      numPts = 0;
      Pts = 0;
      return;
  end;

  % read in the number of points
  numPts = fscanf(RF, '%d', 1);

  % setup Pts
  Pts = zeros(numPts, numVars);

  % loop through to read in the points
  tempR = 0; tempI = 0;
  for k = 1:numPts
      A = fscanf(RF,'%e',2*numVars);
      Pts(k,:) = A(1:2:end) + 1i*A(2:2:end);
      % read in the point
  %    for j = 1:numVars
  %        tempR = fscanf(RF, '%e', 1);
  %        tempI = fscanf(RF, '%e', 1);
  %        Pts(k,j) = tempR + 1i * tempI;
  %    end;
  end;

  % close file
  fclose(RF);

return;


function [VM] = VeroneseMatrix(Pts, deg, normalize)
    % compute the Veronese embedding of 'Pts' in degree 'deg'

    % default
    if nargin < 3
        normalize = 1;
    end;

    N = size(Pts,1); % number of points
    numVars = size(Pts,2); % number of variables

    % initialize VM
    VM = zeros(N,nchoosek(numVars + deg - 1, deg));
%disp(['size: ',num2str(size(VM,1)),' x ',num2str(size(VM,2))]);

%disp('setup degree');
    % setup A
    A = setupDegree(deg, numVars);

    % loop to setup VM
    for j = 1:N
%if mod(j,10000) == 1 disp([num2str(j),' of ',num2str(N)]); end;
        % normalize Pts
        if normalize == 0
            P = Pts(j,:);
        else
            P = Pts(j,:) / norm(Pts(j,:));
        end;
        
        % compute embedding
        if normalize == 0
            VM(j,:) = veronese(P,A);
        else
            v = veronese(P,A);
            VM(j,:) = v/norm(v);
        end;
    end;
return;

function [M] = veronese(X, A)
    % compute the Veronese embedding based on A

    N = size(A,1);
    M = zeros(1,N);
    for j = 1:N
        M(1,j) = prod(X.^A(j,:));
    end;
return;


function [A] = setupDegree(deg, n)
    % setup degrees for Veronese embedding

    numRows = nchoosek(n+deg-1,deg);
    A = zeros(numRows,n);
    A(1,1) = deg;

    for j = 1:numRows-1
        % copy
        A(j+1,:) = A(j,:);
        % find target entry
        target = -1;
        for k = n-1:-1:1
            if A(j+1,k) ~= 0
                target = k;
                break;
            end;
        end;
        % move entry
        A(j+1,target) = A(j+1,target) - 1;
        A(j+1,target+1) = A(j+1,target+1) + 1;
        % move all of final entry to target+1 if not the same!
        if target ~= n-1
            A(j+1,target+1) = A(j+1,target+1) + A(j+1,n);
            A(j+1,n) = 0;
        end;
    end;
return;
